"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncObjectManager = void 0;
/* A react hook for syncing server/client state for a single document.

This was created as a prototype and needs a lot of refactoring!

*/
const react_1 = __importDefault(require("react"));
const merge_1 = __importDefault(require("lodash/merge"));
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const useAsyncFn_1 = __importDefault(require("react-use/lib/useAsyncFn"));
const use_async_fn_reset_1 = __importDefault(require("./use-async-fn-reset"));
const async_hooks_use_async_request_1 = require("@react_db_client/async-hooks.use-async-request");
const helpers_generate_uid_1 = require("@react_db_client/helpers.generate-uid");
const NO_OP_ASYNC = () => __awaiter(void 0, void 0, void 0, function* () { });
const useAsyncObjectManager = ({ activeUid, collection, isNew: isNewIn = false, inputAdditionalData = null, schema = 'all', populate = 'all', loadOnInit = true, autoSave = false, reloadOnSave = false, saveAllOnSave = false, onSavedCallback: onSavedCallbackIn /* Returns a message string */, saveErrorCallback /* Returns a AsyncRequestError */, onDeleteCallback, asyncGetDocument, asyncPutDocument, asyncPostDocument, asyncDeleteDocument, }) => {
    const [isNew, setIsNew] = react_1.default.useState(!activeUid || isNewIn);
    const [uid] = react_1.default.useState(isNew || !activeUid ? (0, helpers_generate_uid_1.generateUid)(collection, null, null) : activeUid);
    const [unsavedChanges, setUnsavedChanges] = react_1.default.useState(false); // TODO: Implement this
    // TODO: Might need to replace with useAsyncFnReset
    const [savingDataState, asyncSaveData, asyncSaveDataReset] = (0, use_async_fn_reset_1.default)(asyncPutDocument, [asyncPutDocument]);
    const [savingNewDataState, asyncSaveNewData, asyncSaveNewDataReset] = (0, use_async_fn_reset_1.default)(asyncPostDocument, [asyncPostDocument]);
    const [deletingDataState, asyncDeleteData, asyncDeleteDataReset] = (0, use_async_fn_reset_1.default)(asyncDeleteDocument || NO_OP_ASYNC);
    const [loadedDataState, callLoadData] = (0, useAsyncFn_1.default)(() => __awaiter(void 0, void 0, void 0, function* () { return asyncGetDocument(collection, uid, schema, populate); }), [collection, uid, schema, populate, asyncGetDocument]);
    const hasLoaded = react_1.default.useRef(false);
    const hasDeleted = react_1.default.useRef(false);
    const [shouldSave, setShouldSave] = react_1.default.useState(false);
    const [newData, setNewData] = react_1.default.useState(loadedDataState.value);
    const [receivedResponse, setReceivedResponse] = react_1.default.useState(false);
    const [savedData, setSavedData] = react_1.default.useState(loadedDataState.value);
    const [combinedData, setCombinedData] = react_1.default.useState(Object.assign(Object.assign(Object.assign({}, (loadedDataState.value || {})), inputAdditionalData), { uid }));
    react_1.default.useEffect(() => {
        if (loadOnInit && !hasLoaded.current) {
            callLoadData();
        }
    }, [loadOnInit, callLoadData]);
    react_1.default.useEffect(() => {
        const error = savingDataState.error || savingNewDataState.error;
        if (receivedResponse && error) {
            setReceivedResponse(false);
            if (saveErrorCallback)
                saveErrorCallback(new async_hooks_use_async_request_1.AsyncRequestError((error === null || error === void 0 ? void 0 : error.message) || 'Unknown Async Request Error', error));
        }
    }, [
        savingDataState,
        savingNewDataState,
        receivedResponse,
        saveErrorCallback,
    ]);
    react_1.default.useEffect(() => {
        if (!deletingDataState.loading &&
            hasDeleted.current &&
            deletingDataState.value) {
            // TODO: Check on delete args
            if (onDeleteCallback)
                onDeleteCallback(deletingDataState.value, [collection, uid]);
        }
    }, [deletingDataState, onDeleteCallback, collection, uid]);
    react_1.default.useEffect(() => {
        if (loadedDataState.value &&
            !loadedDataState.loading &&
            !hasLoaded.current) {
            hasLoaded.current = true;
            setCombinedData(Object.assign(Object.assign(Object.assign({}, (loadedDataState.value || {})), inputAdditionalData), { uid }));
        }
    }, [uid, loadedDataState, inputAdditionalData]);
    react_1.default.useEffect(() => {
        const error = savingDataState.error || savingNewDataState.error;
        const value = savingDataState.value || savingNewDataState.value;
        if (receivedResponse && !error && value) {
            const fullData = Object.assign(Object.assign({}, combinedData), { uid });
            setReceivedResponse(false);
            setIsNew(false);
            setUnsavedChanges(false);
            setSavedData(combinedData);
            if (onSavedCallbackIn)
                onSavedCallbackIn(uid, value, fullData);
            if (reloadOnSave)
                callLoadData();
        }
    }, [
        receivedResponse,
        savingDataState,
        savingNewDataState,
        combinedData,
        uid,
        onSavedCallbackIn,
        reloadOnSave,
    ]);
    react_1.default.useEffect(() => {
        if (shouldSave) {
            setShouldSave(false);
            const dataToSave = saveAllOnSave
                ? Object.assign(Object.assign(Object.assign({}, inputAdditionalData), (combinedData || {})), { uid }) : Object.assign(Object.assign(Object.assign({}, inputAdditionalData), (newData || {})), { uid });
            asyncSaveDataReset();
            asyncSaveNewDataReset();
            const postCall = isNew ? asyncSaveNewData : asyncSaveData;
            postCall(collection, uid, dataToSave).finally(() => {
                setReceivedResponse(true);
            });
        }
    }, [
        uid,
        collection,
        newData,
        shouldSave,
        isNew,
        asyncSaveData,
        asyncSaveNewData,
        asyncSaveDataReset,
        asyncSaveNewDataReset,
        inputAdditionalData,
        combinedData,
    ]);
    const updateField = (field, value, save, nested) => {
        // TODO: Is there a more efficient way to update data here
        setNewData((prev) => {
            let dataCopy = (0, cloneDeep_1.default)(prev);
            if (nested) {
                const nestedLayers = nested.split('.');
                const nestedData = nestedLayers
                    .reverse()
                    .reduce((acc, v) => ({ [v]: acc }), { [field]: value });
                dataCopy = (0, merge_1.default)(dataCopy, nestedData);
            }
            else {
                dataCopy = Object.assign(Object.assign({}, dataCopy), { [field]: value });
            }
            return dataCopy;
        });
        setCombinedData((prev) => {
            let dataCopy = (0, cloneDeep_1.default)(prev);
            if (nested) {
                const nestedLayers = nested.split('.');
                const nestedData = nestedLayers
                    .reverse()
                    .reduce((acc, v) => ({ [v]: acc }), { [field]: value });
                dataCopy = (0, merge_1.default)(dataCopy, nestedData);
            }
            else {
                dataCopy = Object.assign(Object.assign({}, dataCopy), { [field]: value });
            }
            return dataCopy;
        });
        if (save || autoSave)
            setShouldSave(true);
    };
    const updateData = (newData, save) => {
        // TODO: Is there a more efficient way to update data here
        setNewData((prev) => {
            let dataCopy = (0, cloneDeep_1.default)(prev);
            dataCopy = Object.assign(Object.assign({}, dataCopy), newData);
            return dataCopy;
        });
        setCombinedData((prev) => {
            let dataCopy = (0, cloneDeep_1.default)(prev);
            dataCopy = Object.assign(Object.assign({}, dataCopy), newData);
            return dataCopy;
        });
        if (save || autoSave)
            setShouldSave(true);
    };
    const saveData = () => {
        setShouldSave(true);
    };
    const deleteObject = () => {
        hasDeleted.current = true;
        asyncDeleteData(collection, uid);
    };
    const resetData = () => {
        throw new Error('Not Implemented');
    };
    const reload = () => {
        hasLoaded.current = false;
        callLoadData();
    };
    const loadError = loadedDataState.error &&
        new async_hooks_use_async_request_1.AsyncRequestError(loadedDataState.error.message, loadedDataState.error);
    const saveError = savingDataState.error &&
        new async_hooks_use_async_request_1.AsyncRequestError(savingDataState.error.message, savingDataState.error);
    const [callCount, setCallCount] = react_1.default.useState(0);
    react_1.default.useEffect(() => {
        if (loadedDataState.loading)
            setCallCount((prev) => prev + 1);
    }, [loadedDataState.loading]);
    return {
        loadedData: loadedDataState.value,
        saveData,
        updateData,
        updateField,
        resetData,
        reload,
        deleteObject,
        saveResponse: savingDataState.value || savingNewDataState.value,
        deleteResponse: deletingDataState.value,
        loadingData: loadedDataState.loading,
        savingData: savingDataState.loading || savingNewDataState.loading,
        deletingData: deletingDataState.loading,
        data: combinedData,
        savedData,
        uid,
        callCount,
        hasLoaded: !loadedDataState.loading && loadedDataState.value != null,
        loadError,
        saveError,
        unsavedChanges,
        isNew,
    };
};
exports.useAsyncObjectManager = useAsyncObjectManager;
//# sourceMappingURL=use-async-object-manager.js.map